#ifndef __GROUP_H
#define __GROUP_H

#include <dir.h>

// ----Classes---------------------------------------------------------------------------------

_CLASSDEF(TMHGroup)
_CLASSDEF(TMHGroups)
_CLASSDEF(TMHGroupListBox)

// ----Types-----------------------------------------------------------------------------------
//
typedef struct	_TMHGroupData	{

		char			Description[ 60      ];		//	Group description
		char			IconFile   [ MAXPATH ];		//	Path to icon file
        int				IconIndex;					//	Index of icon

		} TMHGroupData;

//	Declare TMHGroup, a TODSortable descendant-------------------------------------------------
//
class TMHGroup: public TODSortable
{

  private:

	TMHGroupData		Header;		//	Header info
    HICON				Icon;		//	Icon handle	

  public:

	//	Con- and Destructors
	//
	 TMHGroup		();
	~TMHGroup		();

	//	Retrieve data members
	//
	inline	virtual	const char	   *GetText			() const	{ return Header.Description;   }

	//	Set data members
	//
	inline			void			SetText			( const char *ADesc ) { strcpy( Header.Description, ADesc ); }

	//	Icon handling
    //
	inline			void			GetIconFile		( char *File ) const { strcpy( File, Header.IconFile );	}
	inline			int				GetIconPos		() const			 { return Header.IconIndex;			}
					HICON			GetIcon			() const;
					void			SetIconData		( const char *File = NULL, int Pos = 0 );
					void			DrawIcon		( HDC DC, int x, int y );

	//	TClassLib
	//
	inline	virtual classType 		isA				() const	{ return 0; 				}
	inline	virtual char 		   *nameOf			() const	{ return "TMHMergeGroup";   }
	inline	virtual hashValueType 	hashValue		() const	{ return 0; 				}
			virtual int 			isEqual			( const Object& ) const;
			virtual int 			isLessThan		( const Object& ) const;
			virtual void 			printOn			( ostream& ) const;

};

//	Declare TMHGroups, a TODSortedArray descendant--------------------------------------------
//
class TMHGroups : public TODSortedArray
{

  public:

	//	Constructor
	//
	TMHGroups() :TODSortedArray( 10, 0, 10 ) {};

	//	Lookup functions
	//
	virtual void 					add				( Object& toAdd );
	virtual	RTMHGroup				GetGroup		( const char *ADesc ) const;

	//	TClassLib
	//
	inline	virtual classType 		isA				() const	{ return 0; 			   }
	inline	virtual char 		   *nameOf			() const	{ return "TMHMergeGroups"; }
	inline	virtual hashValueType 	hashValue		() const	{ return 0; 			   }

};

//	Declare TMHGroupListBox, a TODListBox descendant-----------------------------------------
//
class TMHGroupListBox : public TODListBox
{
  private:

	static WORD		IconBorder,			//	Border around icon
					XFactor;        	//	Move text to right
	WORD            YFactor,			//	Center text
					IconWidth;			//	Width  of Icon
					
  public:

	TMHGroupListBox	( PTWindowsObject AParent, int ResourceId, PTModule AModule = NULL );

	//	Setup
    //
	virtual void	SetupWindow				();

	//	ListBox
	//
	virtual void	Display					( LPDRAWITEMSTRUCT DIS, PTODSortable Item );

};

#endif

