//      MHlist v2.0			Segment 1
//
//		GROUP.CPP			TMHGroup(s)
//
//		(C)opyright 1992	by Matthias Hansen. Alle Rechte vorbehalten.
//
//
#pragma  option -zC_TEXT
#include "mhlist.h"
#pragma  hdrstop

#include <shellapi.h>

// ---- TMHGroup -----------------------------------------------------------------------------
//     ==========
//
// ----Constructor----------------------------------------------------------------------------

TMHGroup::TMHGroup()
{

	//	Init data
	//
	memset( &Header, 0, sizeof(Header) );
	Icon = NULL;

	//	Load icon
	//
	SetIconData();

}

// ----Destructor-----------------------------------------------------------------------------

TMHGroup::~TMHGroup()
{

	//	Free icon ?
	//
	if ( Icon ) {

		//	Yes!
        //
		DestroyIcon( Icon );
		Icon = NULL;

		}

}

// ----SetIconData----------------------------------------------------------------------------

void TMHGroup::SetIconData( const char *File, int Pos )
{

	//	New filename ?
	//
	if ( File ) {

		//	Yes, save data
        //
		strcpy( Header.IconFile, File );
		Header.IconIndex = Pos;

		}
	else if ( !Header.IconFile[0] ) {

		//	No, and header is empty
		//
		GetModuleFileName( TheApp->hInstance, Header.IconFile, sizeof(Header.IconFile) );
		Header.IconIndex = 0;

		}

	//	Icon loaded ?
	//
	if ( Icon ) {

		//	Yes!
        //
		DestroyIcon( Icon );
		Icon = NULL;

		}

	//	Load new icon
	//
	if ( ( Icon = ExtractIcon( TheApp->hInstance, Header.IconFile, Header.IconIndex ) ) == HICON(1) )
    	Icon = NULL;

}

// ----GetIcon--------------------------------------------------------------------------------

HICON TMHGroup::GetIcon() const
{

	//	Is an icon currently loaded
	//
	if ( Icon ) {

		//	Yes.
		//
        return Icon;

		}
	else {

		//	No, return empty icon
		//
		return LoadIcon( NULL, IDI_APPLICATION );

        }

}

// ----DrawIcon-------------------------------------------------------------------------------

void TMHGroup::DrawIcon( HDC DC, int x, int y )
{

	//	Is an icon currently loaded
	//
	if ( Icon ) {

		//	Yes, draw it
		//
		::DrawIcon( DC, x, y, Icon );

		}
	else {

		//	No, use empty icon
		//
		HICON	SysIcon = LoadIcon( NULL, IDI_APPLICATION );

		//	Draw it
        //
		::DrawIcon( DC, x, y, SysIcon );

        }

}

// ----isEqual--------------------------------------------------------------------------------

int TMHGroup::isEqual( const Object& testObject ) const
{
	return strcmp( Header.Description, RTMHGroup(testObject).Header.Description ) == 0;
}

// ----isLessThan-----------------------------------------------------------------------------

int TMHGroup::isLessThan( const Object& testObject ) const
{
	return strcmp( Header.Description, RTMHGroup(testObject).Header.Description ) < 0;
}

// ----printOn--------------------------------------------------------------------------------

void TMHGroup::printOn( ostream& outputStream ) const
{
	outputStream << "Group: " <<  Header.Description << " \n";
}

// ---- TMHGroups ----------------------------------------------------------------------------
//     ===========
//
static int 	IsSameGroup	( const class Object& MyObject, void *pvData );

// ----add------------------------------------------------------------------------------------

void TMHGroups::add( Object& toAdd )
{

	//	Check if it's already in the list
	//
	if ( GetGroup( RTMHGroup(toAdd).GetText() ) == NOOBJECT ) {

		//	Add to TODSortedArray and listbox
		//
		TODSortedArray::add( toAdd );

		}

	//	Else it's a duplicate item.
    //

}

// ----getGroup------------------------------------------------------------Search for Group---

RTMHGroup TMHGroups::GetGroup( const char *ADesc ) const
{

	return RTMHGroup( firstThat( IsSameGroup, (void *)ADesc ) );

}

// ----IsSameGroup---------------------------------------------condFunc to search for Group---

static int IsSameGroup( const class Object& MyObject, void *pvData )
{
	//	Is it the required group ?
	//
	return strcmp( RTMHGroup(MyObject).GetText(), (const char *)pvData ) == 0;
}

// ---- TMHGroupListBox ----------------------------------------------------------------------
//     =================
//
WORD TMHGroupListBox::IconBorder = 4;
WORD TMHGroupListBox::XFactor	 = 2;

// ----TMHGroupListBox------------------------------------------------------------------------

TMHGroupListBox::TMHGroupListBox( PTWindowsObject AParent, int ResourceId, PTModule AModule )
		        :TODListBox     ( AParent, ResourceId, AModule)
{

	//	We need another height
	//
	Height = GetSystemMetrics(SM_CYICON) + (IconBorder*2);

};

// ----SetupWindow----------------------------------------------------------------------------

void TMHGroupListBox::SetupWindow()
{
	HDC					DC;
    TEXTMETRIC			TextMetric;

	//	Action
	//
	TODListBox::SetupWindow();

	//	Add pixels for each side to make room for focus rect and highlight
    //
	IconWidth = GetSystemMetrics(SM_CXICON) + (IconBorder*2);

	//	Determine centering factor
	//
	DC = GetDC         ( HWindow  );
		 GetTextMetrics( DC     , &TextMetric );
	  	 ReleaseDC     ( HWindow,  DC );

	YFactor =  (Height
		     -  TextMetric.tmHeight
			 + (TextMetric.tmDescent/2)
			 + (TextMetric.tmInternalLeading)/2) /2;

}

// ----Display--------------------------------------------------------------------------------

void TMHGroupListBox::Display( LPDRAWITEMSTRUCT DIS, PTODSortable Item )
{
	int					TextX,
						TextY;

	//	Move text over to make room for the icon and
	//	center the text vertically.
	//
	TextX = DIS->rcItem.left + IconWidth + XFactor;
	TextY = DIS->rcItem.top  + YFactor;

	//	Display text
    //
	ExtTextOut( DIS->hDC
			  , TextX
			  , TextY
			  , ETO_OPAQUE
			  , &(DIS->rcItem)
			  , Item->GetText()
			  , strlen(Item->GetText())
			  , NULL
			  );

	//	Paint icon
	//
	PTMHGroup(Item)->DrawIcon( DIS->hDC, DIS->rcItem.left+IconBorder, DIS->rcItem.top+IconBorder );

}

// ----eof GROUP.CPP--------------------------------------------------------------------------