//      MHlist v2.0			Segment 2
//
//		ICO_SEL.CPP			TMHIconSelDlg
//
//		(C)opyright 1992	by Matthias Hansen. Alle Rechte vorbehalten.
//
//
#pragma  option -zC_TEXT_ICOSEL
#include "mhlist.h"
#pragma  hdrstop

#include "ico_sel.h"

// ----Defines---------------------------------------------------------------------------------
//
#define ICONBORDER 		4

// ----TMHIconSelListBox---------------------------------------------------------Constructor---

TMHIconSelListBox::TMHIconSelListBox( PTWindowsObject AParent, int ResourceId, PTModule AModule )
                  :TListBox         ( AParent, ResourceId, AModule)
{

	//	Add pixels for each side to make room for focus rect and highlight
    //
	IconHeight = GetSystemMetrics(SM_CYICON) + (ICONBORDER*2);
	IconWidth  = GetSystemMetrics(SM_CXICON) + (ICONBORDER*2);

};

// ----TMHIconSelDlg-------------------------------------------------------------Constructor---

TMHIconSelDlg::TMHIconSelDlg( PTWindowsObject AParent, char *ASourceFile, int &AnIcon )
		      :TDialog      ( AParent, dlg_IconSelect )
{

	//	Create controls
	//
	ListBox  = new TMHIconSelListBox( this, idd_IcoSelList          );
	FileName = new TEdit            ( this, idd_IcoSelText, MAXPATH );

	//	Retrieve colors etc.
    //
	ColorHighlightText	= GetSysColor( COLOR_HIGHLIGHTTEXT );
	ColorHighlight		= GetSysColor( COLOR_HIGHLIGHT     );
	ColorWindowText		= GetSysColor( COLOR_BTNTEXT       );
	ColorWindow			= GetSysColor( COLOR_BTNFACE       );

	//	Save original pointers
	//
	TheSourceFile =  ASourceFile;
    TheIcon		  = &AnIcon;

	//	Fill edit stuff
	//
	strcpy( SourceFile, TheSourceFile );
	Icon = *TheIcon;

};

// ----SetupWindow-----------------------------------------------------------------------------

void TMHIconSelDlg::SetupWindow()
{

	//	Action
	//
	TDialog::SetupWindow();

	//	Center Dialog
	//
	mh_CenterPopup( HWindow );

	//	Init text box
	//
	FileName->SetText( SourceFile );

	//	Fill icon list
	//
	FillIconList();

}

// ----ShutDownWindow(int)------------------------------------------------------------------------

void TMHIconSelDlg::ShutDownWindow( int ARetValue )
{

	//	Release icon memory!
    //
	ClearIconList();

	//	Ok
	//
    TDialog::ShutDownWindow( ARetValue );

}

// ----ShutDownWindow-----------------------------------------------------------------------------

void TMHIconSelDlg::ShutDownWindow()
{

	//	Release icon memory!
    //
	ClearIconList();

	//	Ok
	//
    TDialog::ShutDownWindow();

}

// ----WMMeasureItem---------------------------------------------------------------------------

void TMHIconSelDlg::WMMeasureItem( RTMessage Msg )
{
	MEASUREITEMSTRUCT	   *ms = LPMEASUREITEMSTRUCT(Msg.LParam);

	if ( ms->CtlID == idd_IcoSelList ) {

		ms->itemWidth  = ListBox->GetWidth ();
		ms->itemHeight = ListBox->GetHeight();

		}

}

// ----WMDrawItem------------------------------------------------------------------------------

void TMHIconSelDlg::WMDrawItem( RTMessage Msg )
{
	LPDRAWITEMSTRUCT 	DIS = LPDRAWITEMSTRUCT(Msg.LParam);
	HICON 				MyIcon;

	if ( DIS->CtlID == idd_IcoSelList && DIS->itemData != -1 ) {
    	
		if ( DIS->itemState & ODS_SELECTED ) {

			SetTextColor( DIS->hDC, ColorHighlightText );
			SetBkColor  ( DIS->hDC, ColorHighlight     );

			} 
		else {

			SetTextColor( DIS->hDC, ColorWindowText );
			SetBkColor  ( DIS->hDC, ColorWindow     );

			}

		//	Display empty text
        //
		ExtTextOut( DIS->hDC
				  , DIS->rcItem.left, DIS->rcItem.top
				  , ETO_OPAQUE
				  , &(DIS->rcItem)
				  , "", 0
				  , NULL
				  );


		//	Retrieve icon handle
        //
		MyIcon = (HICON)LOWORD( DIS->itemData );

		//	Paint it
        //
		DrawIcon( DIS->hDC, DIS->rcItem.left+ICONBORDER, DIS->rcItem.top+ICONBORDER, MyIcon );

		}

}

// ----GetIconCount---------------------------------------Gets number of icons in SourceFile---

int TMHIconSelDlg::GetIconCount()
{
	HICON 			IconCount;
	HCURSOR 		Cursor = SetCursor(LoadCursor(NULL, IDC_WAIT));
	int				Result = 0;

	//	Retrieve icon info
	//
	IconCount = ExtractIcon( GetApplication()->hInstance, SourceFile, 0 );

	//	Ok
	//
	if ( int(IconCount) == 1 ) {

		//	Wrong file type
		//
        //	***NYI***
		//mh_InfoErr( TheModule, this, sth_IcoSelInvalidType );

		}
	else if ( IconCount == NULL ) {

		//	No icons in file
		//
		//	***NYI***
		//mh_InfoErr( TheModule, this, sth_IcoSelEmptyFile );

		}
	else {

		//	Ok, free this icon
		//
		DestroyIcon( IconCount );

		//	And return the total # of icons in the file
		//
		Result = int(ExtractIcon( GetApplication()->hInstance, SourceFile, -1 ) );

		}

	//	Restore cursor
	//
	SetCursor( Cursor );

	//	That's all
	//
    return Result;

}

// ----FillIconList----------------------------------------------------------------------------

BOOL TMHIconSelDlg::FillIconList()
{
	HINSTANCE 		MyInstance = GetApplication()->hInstance;
	HCURSOR 		Cursor     = SetCursor(LoadCursor(NULL, IDC_WAIT));
	HICON 			MyIcon;
	BOOL			Result     = FALSE;
	int				Count;

	//	Retrieve # of icons
	//
	if ( ( Count = GetIconCount() ) > 0 ) {

		//	Loop through icons
		//
		for( int i = 0; i < Count; i++ ) {

			//	Extract icon
			//
			MyIcon = ExtractIcon( MyInstance, SourceFile, i) ;

			//	Store it inside the listbox
			//
			ListBox->AddString( LPSTR(MAKELONG(MyIcon, 0)) );


			} // loop through icons
	
		//	Select current icon
		//
		ListBox->SetSelIndex( Icon );

		//	Ok
		//
		Result = TRUE;

		}

	//	Restore cursor
	//
	SetCursor( Cursor );

	//	That's all
	//
    return Result;

}

// ----ClearIconList---------------------------------------------------------------------------

void TMHIconSelDlg::ClearIconList()
{
	HICON 			MyIcon;
	int				Count = ListBox->GetCount();

	//	Free icons
    //
	for ( int i = 0; i < Count; i++ ) {

		//	Retrieve icon handle
		//
		MyIcon = (HICON)LOWORD( SendDlgItemMsg( idd_IcoSelList, LB_GETITEMDATA, i, 0L ) );
		DestroyIcon( MyIcon );

		}

	//	Clear listbox
    //
	ListBox->ClearList();
}

// ----ResetIconList----------------------------------------------------------------------------

void TMHIconSelDlg::ResetIconList()
{
	char			NewFile[ MAXPATH ],			//	New filename
					OldFile[ MAXPATH ];			//	Old ...

	//	Retrieve current filename
	//
	FileName->GetText( NewFile, sizeof(NewFile) ); 

	//	Compare
	//
	if ( stricmp( SourceFile, NewFile ) == 0 ) {

		//	No change, everything is ok
		//
		return;

		}

	//	Save current sourcefile, because we're using implicit names
	//
	strcpy( OldFile   , SourceFile );
	strcpy( SourceFile, NewFile    ); 

	//	Are there any icons in the new file ?
	//
	if ( GetIconCount() ) {

		//	Yes
		//
		Icon = 0;

		//	Clear current list
		//
		ClearIconList();

		//	Fill the new list
		//
		FillIconList();

		}
	else {

		//	No icons in the file, revert to the original one
		//
		strcpy( SourceFile, OldFile   );
		FileName->SetText( SourceFile );

		}

}

// ----Ok----------------------------------------------------------------Search for filename---

void TMHIconSelDlg::Ok( RTMessage Msg )
{

	//	Copy data into calling functions
	//
    strcpy( TheSourceFile, SourceFile );
	*TheIcon = Icon;

	//	Ok
	//
    TDialog::Ok(Msg);

}

// ----HandleFileName--------------------------------------------------------------------------

void TMHIconSelDlg::HandleFileName( RTMessage Msg )
{

	//	Changed ?
	//
	if ( Msg.LP.Hi == EN_CHANGE ) {

		//	Yes, update box
		//
		ResetIconList();

		}

}

// ----Browse------------------------------------------------------------Search for filename---

void TMHIconSelDlg::Browse( RTMessage )
{
   char 			NewFile[ MAXPATH ];

	//	Fill filename
	//
	strcpy( NewFile, "*.exe" );

	if ( GetApplication()->ExecDialog( new TFileDialog( this, SD_FILEOPEN, NewFile ) ) == IDOK )
		FileName->SetText( NewFile );

}

// ----HandleListBox---------------------------------------------------------------------------

void TMHIconSelDlg::HandleListBox( RTMessage Msg )
{

	//	Check notification
	//
	switch ( Msg.LP.Hi ) {

		//	Double-Click
		//
		case LBN_DBLCLK:

			//	Force OK
			//
			Ok(Msg);
			break;

		//	Selection changed
		//
		case LBN_SELCHANGE:

			//	Save selection
			//
			Icon = ListBox->GetSelIndex();
            break;

	  }

}

// ----eof ICO_SEL.CPP-------------------------------------------------------------------------