//      MHlist v2.0			Segment 1
//
//		MHLIST.CPP			Main
//
//		(C)opyright 1992	by Matthias Hansen. Alle Rechte vorbehalten.
//
//
#pragma  option -zC_TEXT
#include "mhlist.h"
#pragma  hdrstop

#include "about.h"

// ----Global data----------------------------------------------------------------------------
//
PTMHlistApp	TheApp;
PTMHlistDlg	TopWnd;

// ----Static data--------------------------------------------------------------Constructor---
//

// ----TMHlistApp---------------------------------------------------------------Constructor---

TMHlistApp::TMHlistApp  ( LPSTR AName, HINSTANCE AnInstance, HINSTANCE APrevInstance, LPSTR ACmdLine, int ACmdShow )
           :TApplication( AName, AnInstance, APrevInstance, ACmdLine, ACmdShow )
{

	//	Create our (empty) array
	//
    List = new TMHGroups;

}

// ----InitMainWindow------------------------------------------------------------------------
//
//	Create TMHlistDlg as the application's main window.
//
void TMHlistApp::InitMainWindow()
{

	//	Create main window
	//
	MainWindow =
	TopWnd     = new TMHlistDlg( List );

}

// ---- TMHlistDlg ---------------------------------------------------------------------------
//     ============
//
// ----Constructor----------------------------------------------------------------------------

TMHlistDlg::TMHlistDlg( PTMHGroups TheList ) :TDialog(NULL, dlg_Main)
{

	//	Init pointers
	//
	List = TheList;

	//	Associate ListBox
	//
	ListBox = new TMHGroupListBox( this, idd_Items );

}

// ----Destructor-----------------------------------------------------------------------------

TMHlistDlg::~TMHlistDlg()
{

	//	Unregister the listbox
	//
    List->UnregisterListBox();

}

// ----GetWindowClass-------------------------------------------------------------------------
//
//	We have our own icon which is installed here.
//
void TMHlistDlg::GetWindowClass( WNDCLASS& WndClass )
{

	//	Retrieve class defition
	//
	TDialog::GetWindowClass(WndClass);

	//	Change to use BWCC's dlg proc
	//
	WndClass.lpfnWndProc = BWCCDefDlgProc;

	//	Load icon
	//
	WndClass.hIcon = LoadIcon(GetApplication()->hInstance, MakeIntRes(ico_About));
}

// ----SetupWindow----------------------------------------------------------------------------
//
//	Add 'About MHlist' to the system menu.
//
void TMHlistDlg::SetupWindow()
{
	HMENU 				hMenu = GetSystemMenu( HWindow, FALSE );

	//	Action
	//
	TDialog::SetupWindow();

	//	Center MHlist
	//
	mh_CenterPopup( HWindow );

	//	Add the 'About MHlist...' dlgbox to the system menu
	//
	AppendMenu( hMenu, MF_SEPARATOR,           0, NULL               );
	AppendMenu( hMenu, MF_STRING   ,   idm_About, "&About MHlist..." );

	//	Disable the 'Maximize' and 'Size' menu commands
	//
	EnableMenuItem( hMenu, SC_MAXIMIZE, MF_DISABLED | MF_GRAYED | MF_BYCOMMAND );
	EnableMenuItem( hMenu, SC_SIZE    , MF_DISABLED | MF_GRAYED | MF_BYCOMMAND );

	//	Fill the listbox
	//
    List->RegisterListBox( ListBox );

	//	Init buttons
	//
	HandleSelection();

}

// ----WMMeasureItem---------------------------------------------------------WM_MEASUREITEM---

void TMHlistDlg::WMMeasureItem( RTMessage Msg )
{
	LPMEASUREITEMSTRUCT	  	MIS = LPMEASUREITEMSTRUCT(Msg.LParam);

	//	Check for correct listbox
    //
	if ( MIS->CtlID == idd_Items ) {

		ListBox->HandleWMMeasureItem( MIS );

        }

}

// ----WMCompareItem---------------------------------------------------------WM_COMPAREITEM---

void TMHlistDlg::WMCompareItem( RTMessage Msg )
{
	LPCOMPAREITEMSTRUCT	  	CIS = LPCOMPAREITEMSTRUCT(Msg.LParam);

	//	Check for correct listbox
    //
	if ( CIS->CtlID == idd_Items ) {

		Msg.Result = ListBox->HandleWMCompareItem( CIS );

        }

}

// ----WMDrawItem---------------------------------------------------------------WM_DRAWITEM---

void TMHlistDlg::WMDrawItem( RTMessage Msg )
{
	LPDRAWITEMSTRUCT 	DIS = LPDRAWITEMSTRUCT(Msg.LParam);

	if ( DIS->CtlID == idd_Items && DIS->itemData != -1 ) {

		//	Call ListBox
        //
		ListBox->HandleWMDrawItem( DIS );

        }

}

// ----ListHandler-------------------------------------------------------idd_MergeSetupItems---
//
void TMHlistDlg::ListHandler( RTMessage Msg )
{

	//	Handle notification LBN_SELCHANGE
	//
	if ( Msg.LP.Hi == LBN_SELCHANGE ) {

		//	Update buttons
        //
		HandleSelection();

		}

}

// ----Add-----------------------------------------------------------------------idd_BorAdd---
//
void TMHlistDlg::Add( RTMessage )
{
	PTMHGroup		NewGroup;

	//	Create the new group
	//
	if ( ( NewGroup = new TMHGroup ) == NULL )
    	return;

	//	Edit
	//
	if ( GetApplication()->ExecDialog( new TMHItemDlg( NewGroup, List, sth_Add, this ) ) == IDOK ) {

		//	Ok, add to array and listbox
		//
		List->add( *NewGroup );

		}
	else {

		//	Free memory
		//
		delete NewGroup;

    	}

}

// ----Change-----------------------------------------------------------idd_BorMergeSetupChg---
//
void TMHlistDlg::Change( RTMessage )
{
	int				Selection = ListBox->GetSelIndex();

	//	Anything selected ?
	//
	if ( Selection >= 0 ) {

		RTMHGroup		EditGroup = RTMHGroup((*(List))[ Selection ]);

		//	Yes, edit it
        //	(ListBox is updated implicitly)
		//
		GetApplication()->ExecDialog( new TMHItemDlg( &EditGroup, List, sth_Chg, this ) );

		}

}

// ----Delete-----------------------------------------------------------idd_BorMergeSetupDel---
//
void TMHlistDlg::Delete( RTMessage )
{
	int				Selection = ListBox->GetSelIndex();

	//	Anything selected ?
	//
	if ( Selection >= 0 ) {

		RTMHGroup			DeleteGroup = RTMHGroup((*(List))[ Selection ]);

		//	Yes, ask user
		//
        //	***NYI***

		//	Remove from array and listbox
		//
		List->detach( DeleteGroup, TShouldDelete::Delete );

		//	Update buttons
		//
        HandleSelection();

		}

}

// ----Ok-------------------------------------------------------------------------------IDOK---
//
void TMHlistDlg::Ok( RTMessage Msg )
{

	//	That's it
	//
    TDialog::Ok(Msg);

}

// ----SysCommand---------------------------------------------------------------WM_SYSCOMMAND---
//
void TMHlistDlg::SysCommand( RTMessage Msg )
{

	//	Which system command is it ?
	//
	switch( Msg.WParam ) {

		//	About
		//
		case idm_About:

			GetApplication()->ExecDialog( new TMHAboutDialog( this ) );
			break;

		//	Everything else
		//
        default:

			DefWndProc(Msg);
			break;

        }
}

// ----HandleSelection------------------------En-/Disable buttons based on current selection---

void TMHlistDlg::HandleSelection()
{
	BOOL		Flag = FALSE;		//	Assume nothing is selected

	//	Anything selected ?
	//
	if ( ListBox->GetSelCount() ) {

		//	Yes, enable buttons
		//
		Flag = TRUE;

		}

	//	Action
    //
	EnableWindow( GetItemHandle(idd_BorChg), Flag );
	EnableWindow( GetItemHandle(idd_BorDel), Flag );

}

// ----WinMain------------------------------------------------------Application entry point---
//
// Run MHlistApp
//
int PASCAL WinMain( HINSTANCE theInstance, HINSTANCE thePrevInstance, LPSTR theCmdLine, int theCmdShow )
{
	HWND				PrevWindow;				//	Previous window handle

	//	Check if we're already up and running
    //
	if ( ( PrevWindow = FindWindow( "BorDlgMHlist", NULL ) ) != NULL ) {

		//	We're already loaded:
		//	Make us the active window and return
		//
		BringWindowToTop( PrevWindow );
		ShowWindow		( PrevWindow, SW_SHOWNORMAL );
		return          ( 0          );

		}

	//	Main OWL action!
	//
	scope {

		TMHlistApp 	MHlistApp( "MHlist", theInstance, thePrevInstance, theCmdLine, theCmdShow );

		//	Save app pointer
		//
		TheApp = &MHlistApp;

		//	Off we go !!!
		//
		MHlistApp.Run();

		//	Return status
		//
		return MHlistApp.Status;

		}

}

// ----eof mhlist.cpp-------------------------------------------------------------------------
